defmodule DecentralisedBookIndex.Metadata.DBIServer do
  use Ash.Resource,
    otp_app: :decentralised_book_index,
    domain: DecentralisedBookIndex.Metadata,
    data_layer: AshPostgres.DataLayer

  alias DecentralisedBookIndex.Metadata

  postgres do
    table "dbi_servers"
    repo DecentralisedBookIndex.Repo
  end

  actions do
    defaults [:read, :destroy]

    create :create do
      primary? true
      accept [:name, :url]
    end

    update :update do
      primary? true
      accept [:name, :url]
    end

    read :by_id do
      argument :id, :uuid, allow_nil?: false
      get? true
      filter expr(id == ^arg(:id))
    end
  end

  attributes do
    uuid_primary_key :id

    attribute :name, :string do
      allow_nil? false
      public? true
    end

    attribute :url, :string do
      allow_nil? false
      public? true
    end

    timestamps()
  end

  relationships do
    has_many :author, Metadata.Author
    has_many :author_alias_registries, Metadata.AuthorAliasRegistry
    has_many :author_roles, Metadata.AuthorRole

    has_many :books, Metadata.Book
    has_many :bids, Metadata.BookId
    has_many :book_editions_registries, Metadata.BookEditionsRegistry

    has_many :publisher, Metadata.Publisher
  end
end
