defmodule DecentralisedBookIndex.Metadata.Forms.BookFormTest do
  use DecentralisedBookIndex.DataCase, async: true

  import ExUnitProperties

  alias DecentralisedBookIndex.Metadata
  alias DecentralisedBookIndex.Metadata.Book

  describe "create new book via the form" do
    test "just name and description" do
      user = generate(user())
      bids = bids(actor: user)
      author_roles = author_roles(actor: user)

      valid_params = %{
        "title" => "Book",
        "description" => "A cool author",
        "format" => "Paper",
        "language" => "English",
        "page_count" => 600,
        "published" => ~D[2025-03-06],
        "author_roles" => author_roles,
        "bids" => bids
      }

      form = \
        AshPhoenix.Form.for_create(Metadata.Book, :create,
          as: "book",
          actor: user
        )

      assert {:ok, user} = AshPhoenix.Form.submit(form, params: valid_params)
    end
  end

  describe "update existing book via the form" do
    test "update name" do
      user = generate(user())
      book = generate(book())

      valid_params = %{
        "title" => "Another Book",
        "description" => "A cool book",
      }

      form =
        AshPhoenix.Form.for_update(book, :update,
          as: "book",
          actor: user
        )

      assert {:ok, user} = AshPhoenix.Form.submit(form, params: valid_params)
    end

    test "update bids and authors" do
      user = generate(user())
      book = generate(book())
      bids = bids(actor: user)
      author_roles = author_roles(actor: user)

      valid_params = %{
        "author_roles" => author_roles,
        "bids" => bids
      }

      form =
        AshPhoenix.Form.for_update(book, :update,
          as: "book",
          actor: user
        )

      assert {:ok, user} = AshPhoenix.Form.submit(form, params: valid_params)
    end
  end

  describe "validation" do
    property "the book form is valid if the page count > 0" do
      user = generate(user())
      book = generate(book())

      valid_count_generator =
        StreamData.integer()
        |> StreamData.map(fn n -> abs(n)+1 end)

      check all(page_count <- valid_count_generator) do
        valid_params = %{
          "page_count" => page_count,
        }

        form =
          AshPhoenix.Form.for_update(book, :update,
            as: "book",
            actor: user
          )

        assert {:ok, _book} = AshPhoenix.Form.submit(form, params: valid_params)
      end
    end

    property "the book form is invalid if the page count <= 0" do
      user = generate(user())
      book = generate(book())

      invalid_count_generator =
        StreamData.integer()
        |> StreamData.map(fn n -> -abs(n) end)

      check all(page_count <- invalid_count_generator) do
        invalid_params = %{
          "page_count" => page_count,
        }

        form =
          AshPhoenix.Form.for_update(book, :update,
            as: "book",
            actor: user
          )

        assert {:error, _error} = AshPhoenix.Form.submit(form, params: invalid_params)
      end
    end
  end
end
